use std::fs;

use anyhow::Result;
use endcap_sl_software_ri_generator::types;
use itertools::Itertools;

fn main() -> Result<()> {
    env_logger::init();
    log::debug!("logger enabled");

    let xmlfile = fs::read_to_string("./csr.xml")?;
    let doc = roxmltree::Document::parse_with_options(
        &xmlfile,
        roxmltree::ParsingOptions {
            allow_dtd: true,
            nodes_limit: u32::MAX,
        },
    )?;
    log::debug!("Parsed: {:#?}", doc);

    let register_map = types::Module::from_xml_dom(doc.root_element())?;
    log::info!("read: {:?}", register_map);
    log::debug!("read: {:#?}", register_map);

    let files = register_map.generate_code()?;
    if log::log_enabled!(log::Level::Debug) {
        for (path, code) in &files {
            log::debug!("path: {:?}", path);
            log::debug!("{}", prettyplease::unparse(code));
            log::trace!("{}", prettyplease::unparse(code));
        }
    }
    if log::log_enabled!(log::Level::Info) {
        for filepath in files.keys().sorted() {
            log::info!("{}", filepath.display());
        }
    }

    endcap_sl_software_ri_generator::write_to_files(files, &std::path::PathBuf::from("out"))?;

    Ok(())
}
